package bccpbmod;
use File::Basename;
use File::Copy;

# bccpbmod.pm - Common module defining global variables & shared subroutines
#
# Authors:  Brandt Onorato, Steve Seifert, Tony Lee, Rich Ralston, Ray Green
# Last updated:  Steve Seifert, 9/14/06
#		     Jane Huang, 7/15/07  
#		     Xiao Lin Zhang, 7/10/2009
#      Xiao Lin Zhang 28/6/10
#
# (c) Copyright IBM Corp. 2008
#
# This is the common routines used in all the Build scripts.
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 



# Share global variables with other modules

use vars qw($debug);
use vars qw($PROD_env,    $QA_env,    $DEV_env);
use vars qw($rccbuild, $sysout);
          

# Define local variables
$rccbuild = 'rccbuild';
$sysout   = 'SYSOUT=*';

$debug = 1;

$PROD_env   = "PROD";
$QA_env     = "QA";
$DEV_env    = "DEV";


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<  Get path value in profileUSER  >>>>>>>>>>>>>>>>>>>>>>>
# profileUSER contains the location of the build scripts, life cycle, project level  
# variable file, view element level variables file, location of output listing,      
# location of output log, location of perl.exe location.
                                      
sub getprofileUSER{
	my($Path_profileUSER,$delimiter) = @_;
	print "get values in profile USER\n" if $debug;
	
	my %PathHash;
	open(USERFile,$Path_profileUSER) or die "Can't open profileUSER: $!\n";   
	
	#for each line, split the variable on the left column and value on the right column
	while($line = <USERFile>){
		
		if( $line =~/^(\s*\n)/ ){
			#blank line
		}else{
		($PathVar,@PathValueArray) = split(/\s+/,$line);
		$PathValue = join(' ',@PathValueArray);#path/file name may have blanks, need to joint together after split
#		print ("$PathVar=$PathValue\n");
		}
		
		#diagnose if the path has a tail delimiter
		$Size = length($PathValue);
		$position = rindex($PathValue,$delimiter);
		if($position eq ($Size-1)){
			chop $PathValue;# delete the tail delimiter for further processing
		}
		$PathHash{$PathVar} = $PathValue; #store the (variable,value) pair into hash table
	}
  close(USERFile);
  
  return ($PathHash{'PARMBLD_SCRIPTS'},$PathHash{'LIFE_CYCLE'},
          $PathHash{'PROJECT_ED'},$PathHash{'PROJECT_VIEW'},
          $PathHash{'OUTPUT_LIST'},$PathHash{'OUTPUT_LOG'},$PathHash{'PERL_EXE'});
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<  User ID Table  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# getUserID uses a lookup table to translate the users' PC username to their
# mainframe ID.  
#
# This routine could be rewritten query a database for the mainframe ID.

sub getUserID {
	my ($scriptfile,$pcid,$delimiter) = @_;

  printf("*** bccpbmod::getUserID ***\n") if $debug;
	printf( "PCID from getUserID: (%s)\n", $pcid ) if $debug;


# *******************************************************************************
# * The user Id table 'userid' is used in this example. It must be modified to  *
# * contain all authorized ids for performing the parameterized build.  Review  *
# * the document for details of setting the user id table.                      *
# * The location of the user Id table can be set by assigning value to variable *
# * PARMBLD_SCRIPTS in profileUSER, user id locates in the same directory with  *
#	* scripts.																																		*              
# ******************************************************************************* 

  my $hostid= "";       # User ID Table 

  if ($scriptfile) {
   	chomp $scriptfile;
      $hostid = $scriptfile . $delimiter . "userid";        
  } else {
 	print "Please specify the script file path in profileUSER.\n";
 	exit -1;
}
  
  print "userid path:$hostid\n" if $debug;
  
  my $pcname ='';
  my $mvsname ='';
  my $line =''; 
  my $mfid = '';

  #for each line, split the variable on the left column and value on the right column
  open(USERIDS, $hostid) or die "Can't open file $hostid: $!\n";
  while ($line = <USERIDS>) {
  	
  	if( $line =~/^(\s*\n)/ ){
			#white line
		}else{
		($pcname,$mvsname) = split(/\s+/,$line);
#		print ("$pcname=$mvsname\n");
		}
  
      if ($pcname eq $pcid) {
            $mfid = $mvsname;
       print "Get mfid from userid table : $mfid \n";
       last;
      } 
  }
  if($mfid eq ''){
   	  print "Can't get mfid from userid table, check if $hostid is userid table or if $hostid is configured in the right way.\n";
  }
  close(USERIDS);
  return $mfid;  
  
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< setEnvironmentVar >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# setEnvironmentVar returns the attribute values stored in projectED file.  

sub setEnvironmentVar {
    my ($EDfile,$lifecycle,$delimiter) = @_;
    printf("*** bccpbmod::setEnvironmentVar ***\n") if $debug;

    # printf( "  PROD ENV   :(%s)\n", $PROD_env ) if $debug;
    # printf( "  QA   ENV   :(%s)\n", $QA_env ) if $debug;
    # printf( "  DEV  ENV   :(%s)\n", $DEV_env ) if $debug;
       
    # printf( "  SVL HOST    :(%s)\n", $os390_hostname ) if $debug;
    
    #choose which projectED file to open depending on lifecycle
    my $projectED = "";
    if($lifecycle eq $DEV_env){
    	$projectED = $EDfile. $delimiter . "projectED_DV";
    } elsif ($lifecycle eq $PROD_env){
    	$projectED = $EDfile. $delimiter . "projectED_PD";
    } elsif ($lifecycle eq $QA_env){
    	$projectED = $EDfile. $delimiter . "projectED_QA";
    } else {
    	print "Please provide a valid life cycle!";
    }
    
    my %EnvHash;
    open(EnvFile, $projectED) or die "Can't open projectED file: $!\n";
    my $tab = "\t";   # tab by default
    # for each line, split out the variable name and value
    while ($line = <EnvFile>) {
    	if( $line =~/^(\s*\n)/ ){
			#white line
			}else{
			($EnvVar,$EnvValue) = split(/\s+/,$line);#split out the variable name and value
			$EnvHash{$EnvVar} = $EnvValue;							#store the (name,value) pair into hash table
#			print ("$EnvVar=$EnvValue\n");
			}

    }
    close(EnvFile);

# Retrieve the environment information from the projectED file
#
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.                                                                      *
# *******************************************************************************

    return ($EnvHash{'env'},$EnvHash{'db2sys'},$EnvHash{'os390_hostname'},$EnvHash{'os390_port'},$EnvHash{'stage'},
     $EnvHash{'copylib1'},$EnvHash{'copylib2'},$EnvHash{'copylib3'},$EnvHash{'copylib4'},
     $EnvHash{'syslib1'},$EnvHash{'syslib2'},$EnvHash{'syslib3'},
 #    $EnvHash{'syslib4'},$EnvHash{'steplib1'},$EnvHash{'steplib2'},	#optional
 #    $EnvHash{'hostpds_prefix'},$EnvHash{'hostpds_affix'},$EnvHash{'cbllib'},	#optional
     $EnvHash{'loadlib'},$EnvHash{'c390llib'},$EnvHash{'dbrmlib'},$EnvHash{'tgtlib'},$EnvHash{'db2_owner'});
     
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< writeFile >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# writeFile formats and writes a JCL deck to a file

sub writeFile {

    my ( $outputfilename, $compile_1, $text ) = @_;

    print "OUTPUTFILE: $outputfilename \n" if $debug;
    open( FOUT, ">> $outputfilename" ) or die "Unable to open output file $outputfilename\n";

    my $n = length($text);

    my $first = substr( $text, 0, 1 );
    my $last;
    if ( $n > 2 ) {
        $last = substr( $text, $n - 2, 2 );
        if ( $last eq "\n\n" ) {
            chop($text);
        }
    }
    if ( $first eq "\n" ) {
        $text = substr( $text, 1 );
    }

    if ($compile_1) {
        print STDERR $text;
    }
    print FOUT $text;

    close(FOUT);

}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< getAttrs  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# getAttrs retrieves the attributes from the element in name/value pairs and returns
# an array where the array element key is the attribute name which contains the value.

sub getAttrs {
	
	my ($VIEWfile,$path) = @_;	#get the project_VIEW file name to look up the attribute values
	my($filename, $directory, $suffix) = fileparse($path);   

	my($file,$testdir,$ext) = File::Basename::fileparse($filename,qr/\.[^.]*/); 
	print " Get attribultes of file: " .$file. "\n";
	
	print ("***bccpmod::getAttrs***\n") if $debug;
	
# get the list of attributes stored in $path 
	open(ATTR,$VIEWfile) or die "Can't open $VIEWfile: $!\n";
	my %attrHash;		#hash table used to store attributes for $file
	
	$line = <ATTR>;	#read the first line to get the column names
	chomp $line;
	my $tab = "\t";   # tab by default
	@colName = split(/$tab/,$line);	#col 1 can be ignored, col2 is element name, col3+ are attribute names
	
	#for each line, split out the variable name and value
	while($line = <ATTR>){
		chomp $line;
	
		@array = split(/$tab/,$line);	#split out the whole line
	
		if((uc($file) eq $array[1]) or (lc($file) eq $array[1]) ){	#find the corresponding line
			for($i=2;$i<=$#array;$i++){
				$attrHash{$colName[$i]}= $array[$i];	#assign (name,value) pair into hash table
			}
			last;
		}else{
			next;#look up the next line
		}
	}
	close(ATTR);
	
	return %attrHash;

}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< testServer >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# testServer performs a specific rccbuild command that verifies communication
# with the mainframe.

sub testServer {
    my ($os390_hostname, $os390_port) = @_;
#    my $rccbuild = 'rccbuild.exe';

    printf("Testing if the server is up...");

    my $cmd = "$rccbuild -h $os390_hostname\@$os390_port -testServer";

    printf("Executing ($cmd)\n") if $debug;

    my $status = 0;

    $status = rcc_system($compile_1, $cmd);

    printf( "Status was(%d)\n", $status ) if $debug;

    if ( $status == 256 ) {
        die
"\n\nUnable to connect to server $os390_hostname\@$os390_port. Aborting compile. \n";
    }
}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< cleanup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# cleanup removes JCL files from a previous execution

sub cleanup {
    my ($path,$delimiter) = @_;
#    print "$path\n";
    if(-e $path and !(-l $path )){
    	if(-d $path and !(-l $path )){
    		# delete files
    		opendir (DIR , $path) or die "Can not open the dir \"$path\": $!\n";
    		my @files = readdir (DIR);
    		foreach my $file (@files){
    			if("." eq $file or ".." eq $file){
    				next;
    			}
 				
   			my $filename = $path . $delimiter . $file;
    			if(-f $filename){
    				if ($file =~/sysprint$/ or $file =~/SYSPRINT$/ ){	# delete list file
    					unlink ($filename);
    				} elsif($file eq "rccbuild.log"){
    					unlink ($filename);	#delete rccbuild.log, compile/upload.log will not be deleted
    				} elsif($file =~/jcl$/ ){
    					unlink ($filename);	#delete jcl
    				}
    			}elsif(-d $filename){
    				cleanup($filename);
    			}
    		}
    	}else{
           mkdir($path);
        }
        
    }else{
       mkdir($path);
    }
    close DIR;

}
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< cleanup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#  Cleanup parts for parmbld_main
# cleanup_parts removes part of JCL files from a previous execution

sub cleanup_parts {
    my ($logdir,$delimiter) = @_;

    if (-d $logdir) {
        unlink($logdir.$delimiter."lnkbatch.jcl");
    } else {
        mkdir $logdir;
    }
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< rcc_system >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# rcc_system executes an rcc command on the mainframe and returns the status

sub rcc_system {
	my ($compile_1, $cmd) = @_;

	my $rcc_out = "rcc.out";
	my $status = 0;
	$cmd .= " > $rcc_out";

	$status = system($cmd);

	open(RCCOUT, $rcc_out) || die "Can't open $rcc_out: $!\n";
	while (<RCCOUT>) {
		print;
		if ($compile_1) {		# SDS 0801
			print STDERR;
		} else {
			print STDERR if /^The MVS step.*return code is/;
		}
	}
	close RCCOUT;
	unlink($rcc_out);

	return $status;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< pause >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# pause waits for the user to press any key

sub pause {

	print "Press enter to exit.\n";
	my $ignore = <STDIN>;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< printArray >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# printArray the values of an array

sub printArray {

	foreach $name (@_) {
		print "\t$name\n";
	}
	return 0;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< my_die >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# my_die exits a script printing the error message that caused the error to stdout
#
sub my_die {
	die "\nFATAL ERROR: @_ \n";
 exit; 
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< movelist >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# move list file generated by rccbuild to user-designated directory
#
sub movelist{
	my($elem_dir,$listdir,$delimiter) = @_;
    opendir(DIRT, $elem_dir) or die "Can't open file $elem_dir: $!\n";
    my @files = readdir(DIRT);
    foreach $file(@files){
    	if("." eq $file or ".." eq $file){
    		next;
    	}
    	if (-f $elem_dir . $delimiter . $file){
    		#check if it's output listing file
				if ($file =~/sysprint$/ or $file =~/SYSPRINT$/){
    			#print "move output listing file:$file to user-defined listing dir: $listdir\n" if $debug;
    			move($elem_dir . $delimiter . $file, $listdir . $delimiter . $file);
    		}
    	}
    }
    close(DIRT);
}


